#ifndef GAME_H
#define GAME_H

#include "toolbox.h"
#include "ResourceManager.h"
#include "Player.h"
#include "Creep.h"
#include "Trap.h"
#include "Level.h"
#include "Room.h"
#include "Sprite.h"
#include "Input.h"

class Game
{
    public:
        static const int TRANSITION_TIME;
        static const int DESCENT_TIME;
        static const int GAME_OVER_FADE_TIME;
        static const int ROOM_SIZE;

        Game(sf::RenderWindow& window);
        ~Game();

        void resetGame();
        void step();
        void draw();
    private:
        void startGame();
        void mainMenuStep();
        void drawMainMenu();
        void drawGui();
        void handleInput();
        void handleTransition();


        Input input;
        sf::RenderWindow& window;
        sf::RenderTexture buffer;
        bool onBuffer;
        Player* player;
        Level* currLevel;
        int depth;

        unsigned int frame = 0;

        bool transitioning;
        int transitionTimer;
        int descentTimer;

        sf::Vector2i transitionDir;
        Room* currRoom;
        Room* nextRoom;

        bool showBoundingBoxes;
        bool fullScreen;
        //gameover vars
        int gameOverFadeTimer;

        //Menu vars
        bool onMainMenu;
        bool startingGame;
        Sprite menuPlayerSprite;
        Sprite menuPlayerIdleSprite;
        float menuPlayerLocation;
        bool paused;
        float menuShakeAmount;
};

#endif // GAME_H
