#ifndef PLAYER_H
#define PLAYER_H

#include "Entity.h"
#include "Sprite.h"

class Player : public Entity
{
    public:
        static const int HURT_TIME;
        static const float WIDTH;
        static const float HEIGHT;
        static const float MOVE_SPEED;
        static const unsigned MAX_ATTACK_FRAMES;
        static const unsigned FALL_FRAMES;
        Player(float x, float y);
        virtual ~Player();

        virtual bool hurt(int dmg);
        virtual void step();
        virtual void draw(sf::RenderTarget& window);
        void drawStats(sf::RenderTarget& window, int px, int py);
        void drawItemWave(sf::RenderTarget& window, float px, float py);
        virtual void doCollide(Entity* other);

        void handleTransition();


        void moveLeft();
        void moveRight();
        void moveUp();
        void moveDown();
        void attack();
        void startRun();
        void stopRun();
        void fallDownHole();
    private:
        Sprite idleSprite;
        Sprite fallSprite;
        Sprite deadSprite;
        Sprite walkSprite;
        Sprite attackSprite;
        Sprite* currSprite;
        Sprite statSprite;

        bool facingRight;
    public:
        bool attacking;
    private:
        sf::Vector2i direction;

        int hurtTimer;
        bool running;
        bool moving;
        float moveSpeed;
        unsigned attackFrames;
    public:
        bool fallingIntoHole;
        unsigned fallTimer;
        bool transition;
        sf::Vector2i transitionDir;
        int attackPow;
        int attackSpeed;
        int runSpeed;
        int numKeys;
        bool bossKey;
        bool descending;
        float descentPercent;
        bool leaving;
};

#endif // PLAYER_H
