#ifndef RESOURCEMANAGER_H
#define RESOURCEMANAGER_H

#include "toolbox.h"

class ResourceManager
{
    public:
        static const string gfxPath;
        static const string sfxPath;
        static const string musicPath;
        static const string fontPath;
        static const string shaderPath;
    private:
        static map<string, sf::Texture*> textures;
        static map<string, sf::Font*> fonts;
        static map<string, sf::SoundBuffer*> soundBuffers;
        static map<string, sf::Music*> music;
        static map<string, sf::Shader*> shaders;

        static list<sf::Sound*> playingSounds;

        enum File_Sections
        {
            IMAGES_START,
                SPRITES,
                BACKGROUNDS,
                DOODADS,
            IMAGES_END,
            FONTS,
            SOUNDS,
            MUSIC,
            LEVELS,
            NUM_SECTIONS
        };
    public:
        static vector<string> bgNames;
        static vector<string> songNames;
        static vector<string> doodadTextures;
        static vector<string> levelNames;

        static void initialize();
        static void loadAllTextures();
        static void loadAllSounds();
        static void cleanUp();

        static sf::Font& getDefaultFont();
        static sf::Font& getFont(string filename);

        static sf::SoundBuffer& getSoundBuffer(string filename);
        static void playSound(string filename);

        static sf::Music& getMusic(string filename);
        static sf::Music* getMusicPointer(string filename);
        static void playMusic(string filename);
        static void stopMusic(string filename);
        static void stopAllMusic();

        static bool loadTexture(string filename);
        static bool loadFont(string filename);
        static bool loadSound(string filename);
        static bool loadMusic(string filename);
        static bool loadShader(string filename, sf::Shader::Type type);

        static void unloadTexture(string filename);
        static void unloadFont(string filename);
        static void unloadSound(string filename);
        static void unloadMusic(string filename);
        static void unloadShader(string filename);

        static sf::Texture& getTexture(string filename);
        static void playSong(string filename);

        static void setSfxVolume(float volume);
        static void setMusicVolume(float volume);
        static void toggleMute();
        static bool isMuted();

        static sf::Shader* getShaderPointer(string filename);
    private:
        static bool muted;
        static float musicVolume;
        static float sfxVolume;
};

#endif // RESOURCEMANAGER_H
