#ifndef TOOLBOX_H_INCLUDED
#define TOOLBOX_H_INCLUDED

#define PI 3.14159265359
#include <SFML/Graphics.hpp>
#include <SFML/Audio.hpp>
#include <cstdlib>
#include <cstring>
#include <cstdio>
#include <cmath>
#include <string>
#include <sstream>
#include <list>
#include <vector>
#include <queue>
#include <unordered_map>
#include <unordered_set>
#include <stack>
#include <iostream>
#include <fstream>
#include <istream>
#include <iterator>

#include <cassert>


using namespace std;

class estream
{
    public:
       fstream file;
       string path;
       bool deleteAfter;

        estream(string name, bool load, bool deleteAfter = false):
            file(name.c_str(), fstream::out | fstream::in | fstream::binary),
            path(name),
            deleteAfter(deleteAfter)
        {
            if(!file && !load)
            {
                file.clear();
                file.open(name.c_str(), fstream::out | fstream::binary);
                if(!file)
                {
                    printf("%s!\n\tCould not create file!: %s\n", strerror(errno), name.c_str());
                }
                else
                {
                    // Suppress debug output for temp files
                    if(!deleteAfter)
                    {
                        printf("Successfully created file: %s !\n", name.c_str());
                    }
                }
                file.close();
                file.open(name.c_str(), fstream::out | fstream::in | fstream::binary);
            }
        }

        void close()
        {
            file.close();
            if(deleteAfter)
            {
                remove(path.c_str());
            }
        }

        bool is_open()
        {
            return file.is_open();
        }

        ///WRITING

        friend estream& operator << (estream& e, const bool& b)
        {
            e.file.write((char*)&b, sizeof(bool));
            return e;
        }

        friend estream& operator << (estream& e, const int& i)
        {
            e.file.write((char*)&i, sizeof(int));
            return e;
        }

        friend estream& operator << (estream& e, const unsigned long long& l)
        {
            e.file.write((char*)&l, sizeof(unsigned long long));
            return e;
        }

        friend estream& operator << (estream& e, const unsigned int& i)
        {
            e.file.write((char*)&i, sizeof(unsigned int));
            return e;
        }

        friend estream& operator << (estream& e, const string& s)
        {
            int size = s.size();
            e<<size;

            e.file.write(s.c_str(), s.size());

            return e;
        }

        friend estream& operator << (estream& e, const float& f)
        {
            e.file.write((char*)&f, sizeof(float));
            return e;
        }

        ///READING

        friend estream& operator >> (estream& e, bool& b)
        {
            e.file.read((char*)&b, sizeof(bool));
            return e;
        }

        friend estream& operator >> (estream& e, int& i)
        {
            e.file.read((char*)&i, sizeof(int));
            return e;
        }

        friend estream& operator >> (estream& e, unsigned long long& l)
        {
            e.file.read((char*)&l, sizeof(unsigned long long));
            return e;
        }

        friend estream& operator >> (estream& e, unsigned int& i)
        {
            e.file.read((char*)&i, sizeof(unsigned int));
            return e;
        }

        friend estream& operator >> (estream& e, string& s)
        {
            int size;
            e>>size;
            //e.file.read((char*)&size, sizeof(int));

            s.resize(size);

            char* buffer = new char[size];

            e.file.read(buffer, s.size());
            for(int i = 0; i < size; i++)
            {
                s[i] = buffer[i];
            }

            delete[] buffer;
            return e;
        }

        friend estream& operator >> (estream& e, float& f)
        {
            e.file.read((char*)&f, sizeof(float));
            return e;
        }

};
#endif // TOOLBOX_H_INCLUDED
