#include "Boss.h"
#include "Player.h"
#include "Room.h"
#include "CombatParticle.h"
const float Boss::W = 18;
const float Boss::H = 18;
const unsigned Boss::ATTACK_CHARGE = 60;
const unsigned Boss::ATTACK_COOL = 100;
const unsigned Boss::TELEPORT_FRAMES = 50;
const unsigned Boss::IDLE_FRAMES = 60;
const int Boss::HURT_TIME = 30;
Boss::Boss(float x, float y, unsigned difficultyLevel) :
    Entity(BOSS,x,y,W,H),
    difficulty(difficultyLevel),
    behave(BEHAVE_ATTACK),
    attackCharge(ATTACK_CHARGE),
    attackCounter(0),
    idleFrames(IDLE_FRAMES),
    sprIdle(ResourceManager::getTexture("boss_idle.png"), 32, 32, -1, 10, 0, 0, true),
    sprCast(ResourceManager::getTexture("boss_cast.png"), 32, 32, -1, 10, 0, 0, false),
    sprWalk(ResourceManager::getTexture("boss_walk.png"), 32, 32, -1, 10, 0, 0, true),
    sprDead(ResourceManager::getTexture("boss_dead.png"), 32, 32, -1, 10, 0, 0, true),
    currSprite(&sprIdle),
    hurtTimer(0),
    facingRight(false)
{
    hp = maxHp = 3 + difficulty / 2;
    removeOnDie = false;
}
Boss::~Boss()
{
}
bool Boss::hurt(int dmg)
{
    for(unsigned i = 0; i < 10; i++)
    {
        room->addParticle(new Particle(Particle::TYPE_BLOOD, x+w/2, y+h/2));
    }

    hurtTimer = HURT_TIME;
    return Entity::hurt(dmg);
}
void Boss::step()
{
    if(hurtTimer) hurtTimer--;
    Player* player = room->getPlayer();
    switch(behave)
    {
    case BEHAVE_IDLE:
        if(die) break;
        if(idleFrames)
        {
            idleFrames--;
            facingRight = player->x+player->w/2 > x+w/2;
        }
        else
        {
            attackCharge = ATTACK_CHARGE;
            behave = BEHAVE_ATTACK;
        }
        break;
    case BEHAVE_ATTACK:
        if(die) break;
        if(attackCharge)
        {
            attackCharge--;
            facingRight = player->x+player->w/2 > x+w/2;
            if(!attackCharge)
            {
                ResourceManager::playSound("magic_boom.wav");
                sf::Vector2f playerDir((player->x+player->w/2) - (x+w/2),
                                       (player->y+player->h/2) - (y+h/2));
                if(attackCounter%3 == 0 || attackCounter%5 == 0)
                {
                    room->addEntity(new CombatParticle(x+w/2 - CombatParticle::W/2, y+h/2 - CombatParticle::H/2, CombatParticle::COMBAT_DEATH_BALL, playerDir, 1 + difficulty/3));
                }
                else
                {
                    room->addEntity(new CombatParticle(x+w/2 - CombatParticle::W/2, y+h/2 - CombatParticle::H/2, CombatParticle::COMBAT_MAGIC_BALL, playerDir, 1 + difficulty/5));
                }
                attackCool = ATTACK_COOL;
                attackCounter++;
            }
        }
        else if(attackCool)
        {
            attackCool--;
            if(!attackCool)
            {
                float tempX = x;
                float tempY = y;
                while(true)///TROLOLOLOLOLOLOLOLOLOLolololololololol.....
                {
                    x = Room::TILE_SIZE + (room->w-2)*Room::TILE_SIZE*(float)rand()/RAND_MAX;
                    y = Room::TILE_SIZE + (room->h-2)*Room::TILE_SIZE*(float)rand()/RAND_MAX;
                    collideRoom();
                    if(!landedDown && !landedLeft && !landedRight && !landedUp)
                    {
                        break;
                    }
                }
                facingRight = player->x+player->w/2 > x+w/2;
                teleportX = x;
                teleportY = y;
                x = prevX = tempX;
                y = prevY = tempY;
                tpFrames = TELEPORT_FRAMES;
                behave = BEHAVE_TELEPORT;
            }
        }
        break;
    case BEHAVE_TAUNT:
        if(die) break;
        break;
    case BEHAVE_TELEPORT:
        if(die) break;
        if(tpFrames)
        {
            tpFrames--;
            float progress = (float)(TELEPORT_FRAMES-tpFrames)/TELEPORT_FRAMES;
            progress *= progress;
            sprWalk.setSpeed(3+progress*10);
            sf::Vector2f tpVec(teleportX - prevX, teleportY - prevY);
            tpVec *= progress;
            x = prevX + tpVec.x;
            y = prevY + tpVec.y;

            facingRight = tpVec.x > 0;
            if(!tpFrames)
            {
                sprCast.reset();
                attackCharge = ATTACK_CHARGE;
                behave = BEHAVE_ATTACK;
                ResourceManager::playSound("magic_cast.wav");
            }
        }
        break;
    }

    x += dx;
    y += dy;
    dx *= 0.8;
    dy *= 0.8;
}
void Boss::draw(sf::RenderTarget& window)
{
    switch(behave)
    {
    case BEHAVE_IDLE:
        currSprite = &sprIdle;
        break;
    case BEHAVE_ATTACK:
        if(attackCharge)
        {
            currSprite = &sprCast;
        }
        else
        {
            currSprite = &sprIdle;
        }
        break;
    case BEHAVE_TELEPORT:
        currSprite = &sprWalk;
        break;
    default:
        //warning go away plz
        break;
    }
    if(die)
    {
        currSprite = &sprDead;
    }
    else
    {
        currSprite->setFlipped(!facingRight);
    }
    if(hurtTimer > 0)
    {
        float f = 1.f - (float)hurtTimer / HURT_TIME;
        currSprite->setColor(sf::Color(255, 255 * f, 255 * f));
        currSprite->setPosition(x + w/2 + rand()%3 - 1, y + h/2);
    }
    else
    {
       currSprite->setColor(sf::Color::White);
        currSprite->setPosition(x + w/2, y + h/2);
    }
    currSprite->step();
    currSprite->draw(window);
}
void Boss::doCollide(Entity* other)
{
    if(behave == BEHAVE_TELEPORT)
        return;
    if(!collidesWith(other))
        return;
    if(other != room->getPlayer())
        return;
    Player* player = room->getPlayer();
    sf::Vector2f reflector((player->x+player->w/2) - (x+w/2),
                           (player->y+player->h/2) - (y+h/2));
    float reflectorMag = sqrt(pow(reflector.x,2)+pow(reflector.y,2));
    if(reflectorMag > 0) reflector /= reflectorMag;
    reflector *= Player::MOVE_SPEED;
    player->dx += reflector.x;
    player->dy += reflector.y;
}
