#include "BossDoor.h"
#include "Room.h"

const float BossDoor::L = Room::TILE_SIZE*2;
const float BossDoor::S = Room::TILE_SIZE;

BossDoor::BossDoor(float x, float y, bool vertical):
    Entity(DOOR,x,y,(vertical?S:L),(vertical?L:S)),
    vertical(vertical),
    sprite(ResourceManager::getTexture("boss_door.png"), 32, 16, -1, 15, 0, 0, false)
{
    sprite.setOrigin(0, 0);
}

BossDoor::~BossDoor()
{
    //dtor
}

bool BossDoor::hurt(int dmg)
{
    return false;
}

void BossDoor::step()
{
    return;
}

void BossDoor::draw(sf::RenderTarget& window)
{

    if (vertical)
    {
        sprite.setRotation(90);
        sprite.setPosition(x + 16, y);
    }
    else
    {
        sprite.setPosition(x, y);
        sprite.setRotation(0);
    }
    sprite.draw(window);
}

void BossDoor::doCollide(Entity* other)
{
    if (!collidesWith(other)) return;

    Player* player = room->getPlayer();
    if (other == player && player->bossKey)
    {
        die = true;
        player->bossKey = false;
        for(unsigned i = 0; i < 20; i++)
        {
            room->addParticle(new Particle(Particle::TYPE_DUST, x+rand()%(int)w, y+rand()%(int)h));
            room->addParticle(new Particle(Particle::TYPE_SHINY, x+rand()%(int)w, y+rand()%(int)h));
        }
    }
    else
    {
        int resDir = 0;
        int minDiff = y - (other->y+other->h);
        if(fabsf(x - (other->x+other->w)) < fabsf(minDiff))
        {
            minDiff = x - (other->x+other->w);
            resDir = 1;
        }
        if(fabsf((y+h) - other->y) < fabsf(minDiff))
        {
            minDiff = (y+h) - other->y;
            resDir = 2;
        }
        if(fabsf((x+w) - other->x) < fabsf(minDiff))
        {
            minDiff = (x+w) - other->x;
            resDir = 3;
        }
        if(resDir % 2 == 0)
        {
            other->y += minDiff;
        }
        else
        {
            other->x += minDiff;
        }
    }
}
