#include "Entity.h"
#include "Room.h"

Entity::Entity(EntityType entityType, float x, float y, float w, float h):
    entityType(entityType),
    room(nullptr),
    x(x),
    y(y),
    die(false),
    w(w),
    h(h),
    dx(0),
    dy(0),
    hp(1),
    landedDown(false),
    landedLeft(false),
    landedRight(false),
    landedUp(false),
    bouncy(false),
    removeOnDie(true)
{
    //ctor
}

Entity::~Entity()
{
    //dtor
}

void Entity::setRoom(Room* newRoom)
{
    room = newRoom;
}

bool Entity::hurt(int dmg)
{
    hp -= dmg;
    if(hp <= 0)
    {
        die = true;
    }
    return true;
}
void Entity::step()
{
}
void Entity::draw(sf::RenderTarget& window)
{
}
#include "Player.h"
void Entity::drawBoundingBox(sf::RenderTarget& window)
{
    sf::RectangleShape box({w, h});
    box.setPosition(x, y);
    box.setOutlineColor(sf::Color(255, 0, 0, 150));
    box.setOutlineThickness(1.f);
    box.setFillColor(sf::Color::Transparent);
    window.draw(box);
}
bool Entity::collidesWith(Entity* other)
{
    return (x + w > other->x &&
       x < other->x + other->w &&
       y + h > other->y &&
       y < other->y + other->h);
}
bool Entity::isOutOfBounds()
{
    return x+w < 0 ||
            y + h < 0 ||
            x > room->w * Room::TILE_SIZE ||
            y > room->h * Room::TILE_SIZE;
}
void Entity::doCollide(Entity* other)
{
}

void Entity::collideRoom()
{
    int startX = max(0, (int)(x / Room::TILE_SIZE));
    int endX = min(room->w-1, (int)((x+w) / Room::TILE_SIZE));
    int startY = max(0, (int)(y / Room::TILE_SIZE));
    int endY = min(room->h-1, (int)((y+h) / Room::TILE_SIZE));

    float resolve;
    bool horizontal = false;
    float resolveX = 0;
    float resolveY = 0;
    for(int i = startX; i <= endX; i++)
    {
        for(int j = startY; j <= endY; j++)
        {
            Tile& t = room->getTile(i, j);

            float minResolve = Room::TILE_SIZE * 5;
            if(t.left)
            {
                resolve = Room::TILE_SIZE * i - (x + w);
                if(fabs(resolve) < fabs(minResolve))
                {
                    minResolve = resolve;
                    horizontal = true;
                }
            }
            if(t.up)
            {
                resolve = Room::TILE_SIZE * j - (y + h);
                if(fabs(resolve) < fabs(minResolve))
                {
                    minResolve = resolve;
                    horizontal = false;
                }
            }
            if(t.right)
            {
                resolve = (i+1) * Room::TILE_SIZE - x;
                if(fabs(resolve) < fabs(minResolve))
                {
                    minResolve = resolve;
                    horizontal = true;
                }
            }
            if(t.down)
            {
                resolve = (j+1) * Room::TILE_SIZE - y;
                if(fabs(resolve) < fabs(minResolve))
                {
                    minResolve = resolve;
                    horizontal = false;
                }
            }
            if(fabs(minResolve) < Room::TILE_SIZE)
            {
                if(horizontal)
                {
                    resolveX = minResolve;
                }
                else
                {
                    resolveY = minResolve;
                }
            }
        }
    }
    landedDown = false;
    landedUp = false;
    landedLeft = false;
    landedRight = false;

    y += resolveY;
    x += resolveX;

    if(resolveX < 0 && dx > 0)
    {
        if(bouncy)
        {
            dx = -dx;
        }
        else
        {
            dx = 0;
        }
        landedRight = true;
    }
    else if(resolveX > 0 && dx < 0)
    {
        if(bouncy)
        {
            dx = -dx;
        }
        else
        {
            dx = 0;
        }
        landedLeft = true;
    }

    if(resolveY < 0 && dy > 0)
    {
        if(bouncy)
        {
            dy = -dy;
        }
        else
        {
            dy = 0;
        }
        landedDown = true;
    }
    else if(resolveY > 0 && dy < 0)
    {
        if(bouncy)
        {
            dy = -dy;
        }
        else
        {
            dy = 0;
        }
        landedUp = true;
    }
}
