#include "Item.h"
#include "Room.h"
#include "Player.h"

Item::Item(float x, float y, ItemType itemType):
    Entity(ITEM, x, y, 16, 16),
    itemType(itemType),
    sprite(ResourceManager::getTexture("items.png"), 16, 16, -1, 15, 0, 0, true)
{
    sprite.setFrame(itemType);
    sprite.setOrigin(0, 0);
}

Item::~Item()
{
    //dtor
}

void Item::step()
{
    if(rand()%60 == 0)
    {
        room->addParticle(new Particle(Particle::TYPE_SHINY, x + rand()%(int)w, y + rand()%(int)h));
    }
}
void Item::draw(sf::RenderTarget& window)
{
    sprite.setPosition(x, y);
    sprite.draw(window);
}

void Item::doCollide(Entity* other)
{
    if(other == room->getPlayer() && collidesWith(other))
    {
        ResourceManager::playSound("powerup.wav");
        die = true;
        Player* player = room->getPlayer();
        switch(itemType)
        {
        case ITEM_HEALTH:
            player->hp++;
            break;
        case ITEM_ATTACK_SPEED:
            player->attackSpeed++;
            break;
        case ITEM_RUN_SPEED:
            player->runSpeed++;
            break;
        case ITEM_KEY:
            player->numKeys++;
            break;
        case ITEM_BOSS_KEY:
            player->bossKey = true;
            break;
        }
        for(unsigned i = 0; i < 20; i++)
        {
            room->addParticle(new Particle(Particle::TYPE_SHINY, x + rand()%(int)w, y + rand()%(int)h));
        }
    }
}
