#include "Trap.h"
#include "Room.h"
#include "Player.h"
#include "CombatParticle.h"
const float Trap::W = Room::TILE_SIZE;
const float Trap::H = Room::TILE_SIZE;
const unsigned Trap::ARROW_COOLDOWN = 60;
const int Trap::ARROW_DAMAGE = 2;
const unsigned Trap::SPIKE_TRIGGER = 12;
const unsigned Trap::SPIKE_COOLDOWN = 30;
const int Trap::SPIKE_DAMAGE = 3;
const unsigned Trap::HOLE_DURABILITY = 30;
const float Trap::HOLE_INFLUENCE = 0.1;
Trap::Trap(TrapType tt, float x, float y) :
    Entity(TRAP,x,y,W,H),
    trapType(tt),
    arrowCooldown(0),
    physical(false),
    spikeTriggered(false),
    spikeTriggerTimer(SPIKE_TRIGGER),
    spikeCooldown(SPIKE_COOLDOWN),
    holeDurability(HOLE_DURABILITY),
    holeCrumbleSoundFrames(0),
    sprite(ResourceManager::getTexture("traps.png"), 16, 16, -1, 15, 0, 0, true)
{
    sprite.setOrigin(0,0);
    switch(trapType)
    {
    case ARROWTRAP:
        physical = true;
        break;
    default:
        break;
    }
}
Trap::~Trap()
{
}
bool Trap::hurt(int dmg)
{
    return false;
}
void Trap::step()
{
    Player* player = room->getPlayer();
    if(arrowCooldown) arrowCooldown--;
    if(holeCrumbleSoundFrames) holeCrumbleSoundFrames--;
    switch(trapType)
    {
    case ARROWTRAP:
        if(arrowCooldown) break;
        if((player->x <= x + w && player->x + player->w >= x) ||
           (player->y <= y + h && player->y + player->h >= y))
        {
            sf::Vector2f toPlayerVec(player->x + player->w/2 - (x + w/2),
                                     player->y + player->h/2 - (y + h/2));
            room->addEntity(new CombatParticle(x+w/2-CombatParticle::W/2,y+h/2-CombatParticle::H/2,
                                               CombatParticle::COMBAT_ARROW,toPlayerVec,ARROW_DAMAGE));
            arrowCooldown = ARROW_COOLDOWN;
            ResourceManager::playSound("arrow_shot.wav");
        }
        break;
    case SPIKETRAP:
        if(spikeTriggered)
        {
            if(spikeTriggerTimer)
            {
                spikeTriggerTimer--;
                if(spikeTriggerTimer == 0)
                {
                    if(collidesWith(player))
                    {
                        player->hurt(SPIKE_DAMAGE);
                        player->dx = player->dy = 0;
                    }
                    ResourceManager::playSound("spike.wav");
                }
            }
            else
            {
                if(spikeCooldown)
                {
                    spikeCooldown--;
                }
                else
                {
                    spikeTriggered = false;
                    spikeTriggerTimer = SPIKE_TRIGGER;
                    spikeCooldown = SPIKE_COOLDOWN;
                }
            }
        }
        break;
    case HOLETRAP:
        break;
    }
}
void Trap::draw(sf::RenderTarget& window)
{
    switch(trapType)
    {
    case ARROWTRAP:
        if(arrowCooldown)
        {
            sprite.setFrame(3);
        }
        else
        {
            sprite.setFrame(2);
        }
        break;
    case SPIKETRAP:
        if(spikeTriggered && !spikeTriggerTimer)
        {
            sprite.setFrame(1);
        }
        else
        {
            sprite.setFrame(0);
        }
        break;
    case HOLETRAP:
        {
            float dPercent = (float)holeDurability/HOLE_DURABILITY;
            if(dPercent > 0.75)
            {
                sprite.setFrame(4);
            }
            else if(dPercent > 0.5)
            {
                sprite.setFrame(5);
            }
            else if(dPercent > 0.25)
            {
                sprite.setFrame(6);
            }
            else if(dPercent == 0)
            {
                sprite.setFrame(7);
            }
        }
        break;
    }
    sprite.setPosition(x,y);
    sprite.draw(window);
    return;
    ///OBVIOUSLY DEBUG DRAW SHIT...
    sf::RectangleShape rect;
    rect.setSize(sf::Vector2f(w,h));
    rect.setPosition(x,y);
    if(spikeTriggered)
    {
        if(spikeTriggerTimer)
        {
            rect.setFillColor(sf::Color::Red);
        }
        else
        {
            rect.setFillColor(sf::Color::Green);
        }
    }
    if(trapType == HOLETRAP)
    {
        if(holeDurability)
        {
            rect.setFillColor(sf::Color::Green);
        }
        else
        {
            rect.setFillColor(sf::Color::Red);
        }
    }
    window.draw(rect);
}
void Trap::doCollide(Entity* other)
{
    if(!collidesWith(other))
        return;
    if(physical)
    {
        int resDir = 0;
        int minDiff = y - (other->y+other->h);
        if(fabsf(x - (other->x+other->w)) < fabsf(minDiff))
        {
            minDiff = x - (other->x+other->w);
            resDir = 1;
        }
        if(fabsf((y+h) - other->y) < fabsf(minDiff))
        {
            minDiff = (y+h) - other->y;
            resDir = 2;
        }
        if(fabsf((x+w) - other->x) < fabsf(minDiff))
        {
            minDiff = (x+w) - other->x;
            resDir = 3;
        }
        if(resDir % 2 == 0)
        {
            other->y += minDiff;
        }
        else
        {
            other->x += minDiff;
        }
    }
    if(other == room->getPlayer())
    {
        Player* player = room->getPlayer();
        if(trapType == SPIKETRAP)
        {
            spikeTriggered = true;
        }
        if(trapType == HOLETRAP)
        {
            float pMoveSpeed = sqrt(player->dx*player->dx + player->dy*player->dy);
            if(holeDurability && pMoveSpeed > 0.1)
            {
                holeDurability--;
                if(!holeCrumbleSoundFrames || !holeDurability)
                {
                    ResourceManager::playSound("floor_crumble.wav");
                    holeCrumbleSoundFrames = 10;
                }
            }
            if(!holeDurability)
            {
                sf::Vector2f pCenter(player->x+player->w/2, player->y+player->h/2);
                sf::Vector2f holeInfluence((x+w/2) - pCenter.x, (y+h/2) - pCenter.y);
                float holeCenterDist = sqrtf(powf(holeInfluence.x,2) + powf(holeInfluence.y,2));
                //if(pCenter.x >= x && pCenter.x <= x+w && pCenter.y >= y && pCenter.y <= y+h)
                if(holeCenterDist < 6)
                {
                    player->fallDownHole();
                }
                else
                {
                    player->x += player->dx*-0.8;
                    player->y += player->dy*-0.8;
                    if(fabs(holeInfluence.x) > fabs(holeInfluence.y))
                    {
                        holeInfluence.x = (holeInfluence.x > 0) ? 1 : -1;
                        holeInfluence.y = 0;
                    }
                    else
                    {
                        holeInfluence.x = 0;
                        holeInfluence.y = (holeInfluence.y > 0) ? 1 : -1;
                    }
                    holeInfluence *= HOLE_INFLUENCE;
                    player->x += holeInfluence.x;
                    player->y += holeInfluence.y;
                }
            }
        }
    }
}




